# -*- coding: utf8 -*-

import logger
import requests
import time
import wdcommon
from artix.common.servicelock import ServiceLock
from datetime import datetime, timedelta
from time import sleep

log = logger.getLogger()


def checkUnloadRequired(host, port, cashCode):
    """
    Функция получения даты и времени последней выгрузки продаж
    :param host - порт REST сервера
    :param port - порт REST сервера
    :param cashCode - код кассы
    """
    log.info(u"Получение времени последней выгрузки продаж")
    requestUrl = "http://%s:%s/CSrest/rest/cashes/%s/state" % (host, port, cashCode)
    log.info(requestUrl)
    with requests.Session() as session:
        response = session.get(requestUrl, timeout=60)
    log.debug(response.content)
    jsonResponse = response.json()
    errorMessage = u"Ошибка при получении времени последней продажи с сервера"

    if response.status_code == requests.codes.ok and "lastSaleTime" in jsonResponse:
        return True, jsonResponse["lastSaleTime"]
    elif response.status_code == requests.codes.ok and len(jsonResponse) and "message" not in jsonResponse and "errorCode" not in jsonResponse:
        fromDate = datetime.now() - timedelta(weeks=4)
        fromDateStr = datetime.strftime(fromDate, "%Y-%m-%dT%H:%M:%S.000+0000")

        log.warning(u"Не удалось получить время последней продажи. Будет выполнена выгрузка продаж за последние 4 недели")
        return True, fromDateStr
    elif response.status_code == 400 and "message" in jsonResponse and "errorCode" in jsonResponse:
        errorMessage = u"%s: %s" % (errorMessage, jsonResponse["message"])

    raise Exception(errorMessage)


def unloadSales(fromDate, protocolVersion):
    """
    Функция подключение к SyncServer и выполнение функции выгрузки продаж
    Период выгрузки с @param по текущее время
    """
    log.debug(u"Принудительная выгрузка продаж")

    timeWithoutTimeZone = datetime.strptime(fromDate[:-5], '%Y-%m-%dT%H:%M:%S.%f')
    fromDate = timeWithoutTimeZone - timedelta(seconds=time.timezone)
    fromDateStr = datetime.strftime(fromDate, "%Y-%m-%dT%H:%M:%S.0")
    toDate = datetime.now()
    toDateStr = datetime.strftime(toDate, "%Y-%m-%dT%H:%M:%S.0")

    log.info(u"Принудительная выгрузка продаж за период с %s по %s" % (fromDateStr, toDateStr))
    client = wdcommon.getClient()
    client.unload(str(), fromDateStr, toDateStr, None, None, wdcommon.getConfigInfo(int(protocolVersion)))


if __name__ == "__main__":
    serviceLock = ServiceLock('/linuxcash/cash/sync/bin/watchdog-unload-pidfile', log)
    if not serviceLock.lock(True):
        exit(0)

    log.info(u'Запуск принудительной выгрузки продаж')

    reconnectCount, reconnectPeriod = wdcommon.getReconnectParams(log)
    host, port, cashCode, protocolVersion = wdcommon.getSyncProperties(log)

    result = False
    for i in range(0, reconnectCount + 1):
        try:
            unloadRequired, dateFrom = checkUnloadRequired(host, port, cashCode)
            if unloadRequired:
                unloadSales(dateFrom, protocolVersion)
            result = True
            break
        except Exception as e:
            log.error(u"В процессе выполнения операции произошла ошибка: %s" % e)

        if reconnectCount - i == 0:
            break

        log.warning(u"Повторная попытка через %s секунд. Осталось попыток: %s" % (reconnectPeriod, reconnectCount - i))
        sleep(reconnectPeriod)

    if result:
        log.info(u"Операция выполнена")
    else:
        log.warning(u"Не удалось выполнить принудительную выгрузку продаж")

    serviceLock.unlock()
