# -*- coding: utf8 -*-

from artix.common import py2to3utils
from artix.common.py2to3utils import configparser
from syncThrift import CashCommunicationService, ttypes
from thrift.protocol import TBinaryProtocol
from thrift.transport import TTransport
from thrift.transport.TSocket import TSocket
import os
import sys


def getClient():
    """
    Функция создает клиента для подключение к Thrift-серверу SyncServer
    """
    socket = TSocket()
    socket.port = int(7795)
    socket.setTimeout(20000)
    transport = TTransport.TBufferedTransport(socket)
    transport.open()
    return CashCommunicationService.Client(TBinaryProtocol.TBinaryProtocol(transport))


def getConfigInfo(protocolVersion):
    configInfo = ttypes.SyncConfigInfo()
    configInfo.protocolVersion = protocolVersion
    return configInfo


def getReconnectParams(log):
    """ Получение настроек соединения из переданных скрипту параметров """
    try:
        if len(sys.argv) < 3:
            raise Exception('missing argument')
        reconnectCount = max(int(sys.argv[1]), 0)
        reconnectPeriod = max(int(sys.argv[2]), 0)
    except Exception as e:
        reconnectCount = 5
        reconnectPeriod = 20
        log.warning(u"Некорректные параметры. Используются параметры по умолчанию (%s попыток с периодом %s секунд)" %
                 (reconnectCount, reconnectPeriod))
        log.warning(py2to3utils.ensureStr(str(e)))
    return reconnectCount, reconnectPeriod


def getSyncProperties(log):
    try:
        cp = configparser.ConfigParser()
        cp.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../properties/cash.ini'))
        host = cp.get('Properties', 'dictUrlHostCS')
        port = cp.get('Properties', 'cashServerRestPort')
        cashCode = cp.get('Properties', 'cash.code')
        protocolVersion = cp.get('Properties', 'protocolVersion')
    except Exception as e:
        log.warning(u"Не удалось получить значения параметров сервера или код кассы, завершение работы")
        log.warning(py2to3utils.ensureStr(str(e)))
        sys.exit(0)
    return host, port, cashCode, protocolVersion
